@@ -48,16 +48,18 @@ PlayListModel::PlayListModel(QObject *parent)
 
 int PlayListModel::rowCount(const QModelIndex &parent) const
 {
-    if (parent.isValid())
+    if (parent.isValid()) {
         return 0;
+    }
 
     return m_playlist.size();
 }
 
 QVariant PlayListModel::data(const QModelIndex &index, int role) const
 {
-    if (!index.isValid() || m_playlist.empty())
+    if (!index.isValid() || m_playlist.empty()) {
         return QVariant();
+    }
 
     auto playListItem = m_playlist.at(index.row());
     switch (role) {
@@ -190,7 +192,7 @@ void PlayListModel::getHttpItemInfo(const QString &url, int row)
     ytdlProcess->setArguments(QStringList() << QStringLiteral("-j") << url);
     ytdlProcess->start();
 
-    QObject::connect(ytdlProcess, (void(QProcess::*)(int, QProcess::ExitStatus)) & QProcess::finished, this, [=](int, QProcess::ExitStatus) {
+    connect(ytdlProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [=](int, QProcess::ExitStatus) {
         QString json = QString::fromUtf8(ytdlProcess->readAllStandardOutput());
         QString title = QJsonDocument::fromJson(json.toUtf8())[QStringLiteral("title")].toString();
         int duration = QJsonDocument::fromJson(json.toUtf8())[QStringLiteral("duration")].toInt();
@@ -369,7 +371,7 @@ void PlayListModel::getYouTubePlaylist(const QString &path)
     ytdlProcess->setArguments(QStringList() << QStringLiteral("-J") << QStringLiteral("--flat-playlist") << path);
     ytdlProcess->start();
 
-    QObject::connect(ytdlProcess, (void(QProcess::*)(int, QProcess::ExitStatus)) & QProcess::finished, this, [=](int, QProcess::ExitStatus) {
+    connect(ytdlProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [=](int, QProcess::ExitStatus) {
         QString json = QString::fromUtf8(ytdlProcess->readAllStandardOutput());
         QJsonValue entries = QJsonDocument::fromJson(json.toUtf8())[QStringLiteral("entries")];
         QString playlistTitle = QJsonDocument::fromJson(json.toUtf8())[QStringLiteral("title")].toString();
@@ -567,4 +569,11 @@ void PlayListProxyModel::copyFilePath(int row)
     QGuiApplication::clipboard()->setText(item->filePath());
 }
 
+QString PlayListProxyModel::getFilePath(int row)
+{
+    auto model = qobject_cast<PlayListModel *>(sourceModel());
+    auto item = model->getPlayList().at(row);
+    return item->filePath();
+}
+
 #include "moc_playlistmodel.cpp"
