@@ -6,6 +6,7 @@
 
 #include "mpvitem.h"
 
+#include <QCommandLineParser>
 #include <QCryptographicHash>
 #include <QDir>
 #include <QJsonArray>
@@ -20,7 +21,6 @@
 
 #include <KLocalizedString>
 #include <KShell>
-#include <QCommandLineParser>
 
 #include "application.h"
 #include "audiosettings.h"
@@ -28,6 +28,7 @@
 #include "global.h"
 #include "informationsettings.h"
 #include "mpvcontroller.h"
+#include "mpvproperties.h"
 #include "playbacksettings.h"
 #include "subtitlessettings.h"
 #include "track.h"
@@ -48,20 +49,26 @@ MpvItem::MpvItem(QQuickItem *parent)
     , m_subtitleTracksModel{new TracksModel}
     , m_playlistModel{new PlayListModel}
     , m_playlistProxyModel{new PlayListProxyModel}
+    , m_chaptersModel{new ChaptersModel}
+    , m_watchLaterPath{QString(Global::instance()->appConfigDirPath()).append(QStringLiteral("/watch-later/"))}
 {
     m_playlistProxyModel->setSourceModel(m_playlistModel);
-    mpv_observe_property(m_mpv, 0, "media-title", MPV_FORMAT_STRING);
-    mpv_observe_property(m_mpv, 0, "time-pos", MPV_FORMAT_DOUBLE);
-    mpv_observe_property(m_mpv, 0, "time-remaining", MPV_FORMAT_DOUBLE);
-    mpv_observe_property(m_mpv, 0, "duration", MPV_FORMAT_DOUBLE);
-    mpv_observe_property(m_mpv, 0, "pause", MPV_FORMAT_FLAG);
-    mpv_observe_property(m_mpv, 0, "volume", MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "mute", MPV_FORMAT_FLAG);
-    mpv_observe_property(m_mpv, 0, "aid", MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "sid", MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "chapter", MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "secondary-sid", MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "track-list", MPV_FORMAT_NODE);
+    observeProperty(MpvProperties::self()->MediaTitle, MPV_FORMAT_STRING);
+    observeProperty(MpvProperties::self()->Position, MPV_FORMAT_DOUBLE);
+    observeProperty(MpvProperties::self()->Remaining, MPV_FORMAT_DOUBLE);
+    observeProperty(MpvProperties::self()->Duration, MPV_FORMAT_DOUBLE);
+    observeProperty(MpvProperties::self()->Pause, MPV_FORMAT_FLAG);
+    observeProperty(MpvProperties::self()->Volume, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->VolumeMax, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->Mute, MPV_FORMAT_FLAG);
+    observeProperty(MpvProperties::self()->AudioId, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->SubtitleId, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->Width, MPV_FORMAT_NODE);
+    observeProperty(MpvProperties::self()->Height, MPV_FORMAT_NODE);
+    observeProperty(MpvProperties::self()->SecondarySubtitleId, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->Chapter, MPV_FORMAT_INT64);
+    observeProperty(MpvProperties::self()->ChapterList, MPV_FORMAT_NODE);
+    observeProperty(MpvProperties::self()->TrackList, MPV_FORMAT_NODE);
 
     initProperties();
     setupConnections();
@@ -71,7 +78,7 @@ MpvItem::MpvItem(QQuickItem *parent)
     timer->start();
 
     connect(timer, &QTimer::timeout, this, [=]() {
-        if (duration() > 0 && !pause()) {
+        if (finishedLoading() && duration() > 0 && !pause()) {
             if (position() < duration() - 10) {
                 saveTimePosition();
             } else {
@@ -108,44 +115,48 @@ void MpvItem::initProperties()
     //    setProperty(QStringLiteral("terminal"), InformationSettings::mpvLogging());
     //    setProperty(QStringLiteral("msg-level"), QStringLiteral("all=v"));
 
-    QString hwdec = PlaybackSettings::useHWDecoding() ? PlaybackSettings::hWDecoding() : QStringLiteral("no");
-    setProperty(QStringLiteral("hwdec"), hwdec);
-    setProperty(QStringLiteral("sub-auto"), QStringLiteral("exact"));
-    setProperty(QStringLiteral("volume-max"), QStringLiteral("100"));
-    // set ytdl_path to yt-dlp or fallback to youtube-dl
-    setProperty(QStringLiteral("script-opts"), QStringLiteral("ytdl_hook-ytdl_path=%1").arg(Application::youtubeDlExecutable()));
+    // lockManager is connected to the pauseChanged signal
+    // by setting `pause` to true it ensures the signal is triggered at startup
+    setProperty(MpvProperties::self()->Pause, true);
 
-    setProperty(QStringLiteral("sub-use-margins"), SubtitlesSettings::allowOnBlackBorders() ? QStringLiteral("yes") : QStringLiteral("no"));
-    setProperty(QStringLiteral("sub-ass-force-margins"), SubtitlesSettings::allowOnBlackBorders() ? QStringLiteral("yes") : QStringLiteral("no"));
+    QString hwdec = PlaybackSettings::useHWDecoding() ? PlaybackSettings::hWDecoding() : QStringLiteral("no");
+    setProperty(MpvProperties::self()->HardwareDecoding, hwdec);
+    setProperty(MpvProperties::self()->VolumeMax, QStringLiteral("100"));
+
+    // set ytdl_path to yt-dlp or fallback to youtube-dl
+    setProperty(MpvProperties::self()->ScriptOpts, QStringLiteral("ytdl_hook-ytdl_path=%1").arg(Application::youtubeDlExecutable()));
     QCommandLineParser *cmdParser = Application::instance()->parser();
     QString ytdlFormat = PlaybackSettings::ytdlFormat();
     if (cmdParser->isSet(QStringLiteral("ytdl-format-selection"))) {
         ytdlFormat = cmdParser->value(QStringLiteral("ytdl-format-selection"));
     }
-    setProperty(QStringLiteral("ytdl-format"), ytdlFormat);
+    setProperty(MpvProperties::self()->YtdlFormat, ytdlFormat);
 
-    setProperty(QStringLiteral("sub-font"), SubtitlesSettings::fontFamily());
-    setProperty(QStringLiteral("sub-font-size"), SubtitlesSettings::fontSize());
-    setProperty(QStringLiteral("sub-color"), SubtitlesSettings::fontColor());
-    setProperty(QStringLiteral("sub-shadow-color"), SubtitlesSettings::shadowColor());
-    setProperty(QStringLiteral("sub-shadow-offset"), SubtitlesSettings::shadowOffset());
-    setProperty(QStringLiteral("sub-border-color"), SubtitlesSettings::borderColor());
-    setProperty(QStringLiteral("sub-border-size"), SubtitlesSettings::borderSize());
-    setProperty(QStringLiteral("sub-bold"), SubtitlesSettings::isBold());
-    setProperty(QStringLiteral("sub-italic"), SubtitlesSettings::isItalic());
+    setProperty(MpvProperties::self()->SubtitleAuto, QStringLiteral("exact"));
+    setProperty(MpvProperties::self()->SubtitleUseMargins, SubtitlesSettings::allowOnBlackBorders());
+    setProperty(MpvProperties::self()->SubtitleAssForceMargins, SubtitlesSettings::allowOnBlackBorders());
+    setProperty(MpvProperties::self()->SubtitleFont, SubtitlesSettings::fontFamily());
+    setProperty(MpvProperties::self()->SubtitleFontSize, SubtitlesSettings::fontSize());
+    setProperty(MpvProperties::self()->SubtitleColor, SubtitlesSettings::fontColor());
+    setProperty(MpvProperties::self()->SubtitleShadowColor, SubtitlesSettings::shadowColor());
+    setProperty(MpvProperties::self()->SubtitleShadowOffset, SubtitlesSettings::shadowOffset());
+    setProperty(MpvProperties::self()->SubtitleBorderColor, SubtitlesSettings::borderColor());
+    setProperty(MpvProperties::self()->SubtitleBorderSize, SubtitlesSettings::borderSize());
+    setProperty(MpvProperties::self()->SubtitleBold, SubtitlesSettings::isBold());
+    setProperty(MpvProperties::self()->SubtitleItalic, SubtitlesSettings::isItalic());
 
-    setProperty(QStringLiteral("screenshot-template"), VideoSettings::screenshotTemplate());
-    setProperty(QStringLiteral("screenshot-format"), VideoSettings::screenshotFormat());
+    setProperty(MpvProperties::self()->ScreenshotTemplate, VideoSettings::screenshotTemplate());
+    setProperty(MpvProperties::self()->ScreenshotFormat, VideoSettings::screenshotFormat());
 
-    setProperty(QStringLiteral("audio-client-name"), QStringLiteral("haruna"));
+    setProperty(MpvProperties::self()->AudioClientName, QStringLiteral("haruna"));
     const QVariant preferredAudioTrack = AudioSettings::preferredTrack();
-    setProperty(QStringLiteral("aid"), preferredAudioTrack == 0 ? QStringLiteral("auto") : preferredAudioTrack);
-    setProperty(QStringLiteral("alang"), AudioSettings::preferredLanguage());
+    setProperty(MpvProperties::self()->AudioId, preferredAudioTrack == 0 ? QStringLiteral("auto") : preferredAudioTrack);
+    setProperty(MpvProperties::self()->AudioLanguage, AudioSettings::preferredLanguage());
 
     const QVariant preferredSubTrack = SubtitlesSettings::preferredTrack();
-    setProperty(QStringLiteral("sid"), preferredSubTrack == 0 ? QStringLiteral("auto") : preferredSubTrack);
-    setProperty(QStringLiteral("slang"), SubtitlesSettings::preferredLanguage());
-    setProperty(QStringLiteral("sub-file-paths"), SubtitlesSettings::subtitlesFolders().join(QStringLiteral(":")));
+    setProperty(MpvProperties::self()->SubtitleId, preferredSubTrack == 0 ? QStringLiteral("auto") : preferredSubTrack);
+    setProperty(MpvProperties::self()->SubtitleLanguage, SubtitlesSettings::preferredLanguage());
+    setProperty(MpvProperties::self()->SubtitleFilePaths, SubtitlesSettings::subtitlesFolders().join(QStringLiteral(":")));
 }
 
 void MpvItem::setupConnections()
@@ -166,19 +177,27 @@ void MpvItem::setupConnections()
     connect(m_mpvController, &MpvController::videoReconfig,
             this, &MpvItem::videoReconfig, Qt::QueuedConnection);
 
+    connect(m_mpvController, &MpvController::asyncReply,
+            this, &MpvItem::onAsyncReply, Qt::QueuedConnection);
+
+    connect(this, &MpvItem::currentUrlChanged, this, [=]() {
+        setFinishedLoading(false);
+    });
+
     connect(this, &MpvItem::fileLoaded, this, [=]() {
-        if (!getProperty(QStringLiteral("vid")).toBool()) {
-            command(QStringList{QStringLiteral("video-add"), VideoSettings::defaultCover()});
+        if (!getProperty(MpvProperties::self()->VideoId).toBool()) {
+            commandAsync(QStringList{QStringLiteral("video-add"), VideoSettings::defaultCover()});
         }
 
+        m_chaptersList = getProperty(MpvProperties::self()->ChapterList).toList();
         setWatchLaterPosition(loadTimePosition());
 
         if (m_playlistModel->rowCount() <= 1 && PlaylistSettings::repeat()) {
-            setProperty(QStringLiteral("loop-file"), QStringLiteral("inf"));
+            setProperty(MpvProperties::self()->LoopFile, QStringLiteral("inf"));
         }
 
-        setProperty(QStringLiteral("ab-loop-a"), QStringLiteral("no"));
-        setProperty(QStringLiteral("ab-loop-b"), QStringLiteral("no"));
+        setProperty(MpvProperties::self()->ABLoopA, QStringLiteral("no"));
+        setProperty(MpvProperties::self()->ABLoopB, QStringLiteral("no"));
 
         // this is only run when reloading the last file in the playlist
         // due to the playlist repeat setting being turned off
@@ -186,28 +205,48 @@ void MpvItem::setupConnections()
             setPause(true);
             setPosition(0);
             setIsFileReloaded(false);
+            setFinishedLoading(true);
             return;
         }
 
         if (PlaybackSettings::seekToLastPosition()) {
             setPause(!PlaybackSettings::playOnResume() && watchLaterPosition() > 0);
-            setPosition(watchLaterPosition());
+            setPropertyAsync(MpvProperties::self()->Position,
+                             watchLaterPosition(),
+                             static_cast<int>(AsyncIds::FinishedLoading));
         }
     });
 
     connect(this, &MpvItem::positionChanged, this, [this]() {
-        int pos = m_position;
-        double duration = m_duration;
+        int pos = position();
+        double duration = getCachedPropertyValue(MpvProperties::self()->Duration).toDouble();
         if (!m_secondsWatched.contains(pos)) {
             m_secondsWatched << pos;
             setWatchPercentage(m_secondsWatched.count() * 100 / duration);
         }
     });
 
+    connect(this, &MpvItem::chapterChanged,
+            this, &MpvItem::onChapterChanged);
+
     connect(m_playlistModel, &PlayListModel::playingItemChanged, this, [=]() {
         loadFile(m_playlistModel->getPath());
     });
 
+    connect(this, &MpvItem::chapterListChanged, this, [=]() {
+        const auto chapters = getCachedPropertyValue(MpvProperties::self()->ChapterList);
+        QList<Chapter> chaptersList;
+        for (const auto &chapter : chapters.toList()) {
+            Chapter c = {
+                .title = chapter.toMap()[QStringLiteral("title")].toString(),
+                .startTime = chapter.toMap()[QStringLiteral("time")].toDouble(),
+            };
+            chaptersList.append(c);
+        }
+        m_chaptersModel->setChapters(chaptersList);
+    });
+
+
 #if defined(Q_OS_UNIX)
     // register mpris dbus service
     QString mspris2Name(QStringLiteral("org.mpris.MediaPlayer2.haruna"));
@@ -238,7 +277,7 @@ void MpvItem::setupConnections()
         Q_EMIT playPrevious();
     });
     connect(mp2Player, &MediaPlayer2Player::seek, this, [=](int offset) {
-        command(QStringList() << QStringLiteral("add") << QStringLiteral("time-pos") << QString::number(offset));
+        commandAsync(QStringList() << QStringLiteral("add") << QStringLiteral("time-pos") << QString::number(offset));
     });
     connect(mp2Player, &MediaPlayer2Player::openUri, this, [=](const QString &uri) {
         Q_EMIT openUri(uri);
@@ -260,250 +299,87 @@ void MpvItem::setupConnections()
     // clang-format on
 }
 
-PlayListModel *MpvItem::playlistModel()
-{
-    return m_playlistModel;
-}
-
-void MpvItem::setPlaylistModel(PlayListModel *model)
-{
-    m_playlistModel = model;
-}
-
-PlayListProxyModel *MpvItem::playlistProxyModel()
-{
-    return m_playlistProxyModel;
-}
-
-void MpvItem::setPlaylistProxyModel(PlayListProxyModel *model)
-{
-    m_playlistProxyModel = model;
-}
-
-bool MpvItem::isFileReloaded() const
-{
-    return m_isFileReloaded;
-}
-
-void MpvItem::setIsFileReloaded(bool _isFileReloaded)
-{
-    if (m_isFileReloaded == _isFileReloaded) {
-        return;
-    }
-    m_isFileReloaded = _isFileReloaded;
-    Q_EMIT isFileReloadedChanged();
-}
-
-QString MpvItem::mediaTitle()
-{
-    return getCachedPropertyValue(QStringLiteral("media-title")).toString();
-}
-
-double MpvItem::position()
-{
-    return getCachedPropertyValue(QStringLiteral("time-pos")).toDouble();
-}
-
-void MpvItem::setPosition(double value)
-{
-    if (value == position()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("time-pos"), value);
-}
-
-double MpvItem::watchLaterPosition() const
-{
-    return m_watchLaterPosition;
-}
-
-void MpvItem::setWatchLaterPosition(double _watchLaterPosition)
-{
-    if (qFuzzyCompare(m_watchLaterPosition + 1, _watchLaterPosition + 1)) {
-        return;
-    }
-    m_watchLaterPosition = _watchLaterPosition;
-    Q_EMIT watchLaterPositionChanged();
-}
-
-double MpvItem::remaining()
-{
-    return getCachedPropertyValue(QStringLiteral("time-remaining")).toDouble();
-}
-
-double MpvItem::duration()
-{
-    return getCachedPropertyValue(QStringLiteral("duration")).toDouble();
-}
-
-bool MpvItem::pause()
-{
-    return getCachedPropertyValue(QStringLiteral("pause")).toBool();
-}
-
-void MpvItem::setPause(bool value)
-{
-    if (value == pause()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("pause"), value);
-}
-
-int MpvItem::volume()
-{
-    return getCachedPropertyValue(QStringLiteral("volume")).toInt();
-}
-
-void MpvItem::setVolume(int value)
-{
-    if (value == volume()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("volume"), value);
-}
-
-bool MpvItem::mute()
-{
-    return getCachedPropertyValue(QStringLiteral("mute")).toBool();
-}
-
-void MpvItem::setMute(bool value)
-{
-    if (value == mute()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("mute"), value);
-}
-
-int MpvItem::chapter()
-{
-    return getCachedPropertyValue(QStringLiteral("chapter")).toInt();
-}
-
-void MpvItem::setChapter(int value)
-{
-    if (value == chapter()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("chapter"), value);
-}
-
-int MpvItem::audioId()
-{
-    return getCachedPropertyValue(QStringLiteral("aid")).toInt();
-}
-
-void MpvItem::setAudioId(int value)
-{
-    if (value == audioId()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("aid"), value);
-}
-
-int MpvItem::subtitleId()
-{
-    return getCachedPropertyValue(QStringLiteral("sid")).toInt();
-}
-
-void MpvItem::setSubtitleId(int value)
-{
-    if (value == subtitleId()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("sid"), value);
-}
-
-int MpvItem::secondarySubtitleId()
-{
-    return getCachedPropertyValue(QStringLiteral("secondary-sid")).toInt();
-}
-
-void MpvItem::setSecondarySubtitleId(int value)
-{
-    if (value == secondarySubtitleId()) {
-        return;
-    }
-    Q_EMIT setMpvProperty(QStringLiteral("secondary-sid"), value);
-}
-
 void MpvItem::onPropertyChanged(const QString &property, const QVariant &value)
 {
-    if (property == QStringLiteral("media-title")) {
+    if (property == MpvProperties::self()->MediaTitle) {
         cachePropertyValue(property, value);
         Q_EMIT mediaTitleChanged();
 
-    } else if (property == QStringLiteral("time-pos")) {
+    } else if (property == MpvProperties::self()->Position) {
         cachePropertyValue(property, value);
         m_formattedPosition = Application::formatTime(value.toDouble());
         Q_EMIT positionChanged();
 
-    } else if (property == QStringLiteral("time-remaining")) {
+    } else if (property == MpvProperties::self()->Remaining) {
         cachePropertyValue(property, value);
         m_formattedRemaining = Application::formatTime(value.toDouble());
         Q_EMIT remainingChanged();
 
-    } else if (property == QStringLiteral("duration")) {
+    } else if (property == MpvProperties::self()->Duration) {
         cachePropertyValue(property, value);
         m_formattedDuration = Application::formatTime(value.toDouble());
         Q_EMIT durationChanged();
 
-    } else if (property == QStringLiteral("pause")) {
+    } else if (property == MpvProperties::self()->Pause) {
         cachePropertyValue(property, value);
         Q_EMIT pauseChanged();
 
-    } else if (property == QStringLiteral("volume")) {
+    } else if (property == MpvProperties::self()->Volume) {
         cachePropertyValue(property, value);
         Q_EMIT volumeChanged();
 
-    } else if (property == QStringLiteral("mute")) {
+    } else if (property == MpvProperties::self()->VolumeMax) {
+        cachePropertyValue(property, value);
+        Q_EMIT volumeMaxChanged();
+
+    } else if (property == MpvProperties::self()->Mute) {
         cachePropertyValue(property, value);
         Q_EMIT muteChanged();
 
-    } else if (property == QStringLiteral("chapter")) {
+    } else if (property == MpvProperties::self()->Chapter) {
         cachePropertyValue(property, value);
         Q_EMIT chapterChanged();
 
-    } else if (property == QStringLiteral("aid")) {
+    } else if (property == MpvProperties::self()->ChapterList) {
+        cachePropertyValue(property, value);
+        Q_EMIT chapterListChanged();
+
+    } else if (property == MpvProperties::self()->AudioId) {
         cachePropertyValue(property, value);
         Q_EMIT audioIdChanged();
 
-    } else if (property == QStringLiteral("sid")) {
+    } else if (property == MpvProperties::self()->SubtitleId) {
         cachePropertyValue(property, value);
         Q_EMIT subtitleIdChanged();
 
-    } else if (property == QStringLiteral("secondary-sid")) {
+    } else if (property == MpvProperties::self()->SecondarySubtitleId) {
         cachePropertyValue(property, value);
         Q_EMIT secondarySubtitleIdChanged();
 
-    } else if (property == QStringLiteral("track-list")) {
+    } else if (property == MpvProperties::self()->Width) {
+        cachePropertyValue(property, value);
+        Q_EMIT videoWidthChanged();
+
+    } else if (property == MpvProperties::self()->Height) {
+        cachePropertyValue(property, value);
+        Q_EMIT videoHeightChanged();
+
+    } else if (property == MpvProperties::self()->TrackList) {
         loadTracks();
-
-    } else {
-        // nothing
     }
 }
 
-double MpvItem::watchPercentage()
-{
-    return m_watchPercentage;
-}
-
-void MpvItem::setWatchPercentage(double value)
-{
-    if (m_watchPercentage == value) {
-        return;
-    }
-    m_watchPercentage = value;
-    Q_EMIT watchPercentageChanged();
-}
-
 void MpvItem::loadFile(const QString &file)
 {
-    m_currentFile = file;
-    command(QStringList() << QStringLiteral("loadfile") << m_currentFile);
+    auto url = QUrl::fromUserInput(file);
+    if (m_currentUrl != url) {
+        m_currentUrl = url;
+        Q_EMIT currentUrlChanged();
+    }
 
-    GeneralSettings::setLastPlayedFile(m_currentFile);
+    commandAsync(QStringList() << QStringLiteral("loadfile") << m_currentUrl.toString());
+
+    GeneralSettings::setLastPlayedFile(m_currentUrl.toString());
     GeneralSettings::self()->save();
 }
 
@@ -517,7 +393,7 @@ void MpvItem::loadTracks()
     none->setTitle(i18nc("@action The \"None\" subtitle track is used to clear/unset selected track", "None"));
     m_subtitleTracks.insert(0, none);
 
-    const QList<QVariant> tracks = getProperty(QStringLiteral("track-list")).toList();
+    const QList<QVariant> tracks = getProperty(MpvProperties::self()->TrackList).toList();
     int subIndex = 1;
     int audioIndex = 0;
     for (const auto &track : tracks) {
@@ -565,14 +441,58 @@ void MpvItem::loadTracks()
     Q_EMIT subtitleTracksModelChanged();
 }
 
-TracksModel *MpvItem::subtitleTracksModel() const
+void MpvItem::onAsyncReply(const QVariant &data, int id)
 {
-    return m_subtitleTracksModel;
+    switch (static_cast<AsyncIds>(id)) {
+    case AsyncIds::None: {
+        break;
+    }
+    case AsyncIds::FinishedLoading: {
+        setFinishedLoading(true);
+        break;
+    }
+    case AsyncIds::SavePosition: {
+        auto hash = md5(currentUrl().toLocalFile());
+        auto watchLaterConfig = QString(m_watchLaterPath).append(hash);
+        Q_EMIT syncConfigValue(watchLaterConfig, QString(), QStringLiteral("TimePosition"), data.toString());
+        break;
+    }
+    case AsyncIds::Screenshot: {
+        auto filename = data.toMap().value(QStringLiteral("filename")).toString();
+        if (filename.isEmpty()) {
+            osdMessage(i18nc("@info:tooltip osd", "Screenshot taken"));
+        } else {
+            osdMessage(i18nc("@info:tooltip osd", "Screenshot: %1", filename));
+        }
+        break;
+    }
+    }
 }
 
-TracksModel *MpvItem::audioTracksModel() const
+void MpvItem::onChapterChanged()
 {
-    return m_audioTracksModel;
+    if (!finishedLoading() || !PlaybackSettings::skipChapters()) {
+        return;
+    }
+
+    const QString chaptersToSkip = PlaybackSettings::chaptersToSkip();
+    if (m_chaptersList.count() == 0 || chaptersToSkip == QString()) {
+        return;
+    }
+
+    const QStringList words = chaptersToSkip.split(QStringLiteral(","));
+    auto ch = m_chaptersList.value(chapter()).toMap();
+    auto title = ch.value(QStringLiteral("title")).toString();
+    for (int i = 0; i < words.count(); ++i) {
+        QString word = words.at(i).toLower().simplified();
+        if (!ch.isEmpty() && title.toLower().contains(word)) {
+            commandAsync({QStringLiteral("add"), QStringLiteral("chapter"), QStringLiteral("1")});
+            if (PlaybackSettings::showOsdOnSkipChapters()) {
+                Q_EMIT osdMessage(i18nc("@info:tooltip osd", "Skipped chapter: %1", title));
+            }
+            return;
+        }
+    }
 }
 
 void MpvItem::saveTimePosition()
@@ -582,15 +502,11 @@ void MpvItem::saveTimePosition()
         return;
     }
     // position is saved only for files longer than PlaybackSettings::minDurationToSavePosition()
-    if (getCachedPropertyValue(QStringLiteral("duration")).toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
+    if (getCachedPropertyValue(MpvProperties::self()->Duration).toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
         return;
     }
 
-    auto hash = md5(getProperty(QStringLiteral("path")).toString());
-    auto configPath = Global::instance()->appConfigDirPath();
-    configPath.append(QStringLiteral("/watch-later/")).append(hash);
-
-    Q_EMIT syncConfigValue(configPath, QString(), QStringLiteral("TimePosition"), getCachedPropertyValue(QStringLiteral("time-pos")));
+    m_mpvController->getPropertyAsync(MpvProperties::self()->Position, static_cast<int>(AsyncIds::SavePosition));
 }
 
 double MpvItem::loadTimePosition()
@@ -602,23 +518,24 @@ double MpvItem::loadTimePosition()
     // position is saved only for files longer than PlaybackSettings::minDurationToSavePosition()
     // but there can be cases when there is a saved position for files lower than minDurationToSavePosition()
     // when minDurationToSavePosition() was increased after position was already saved
-    if (getProperty(QStringLiteral("duration")).toInt() < PlaybackSettings::minDurationToSavePosition() * 60) {
+    auto duration = getProperty(MpvProperties::self()->Duration).toInt();
+    if (duration < PlaybackSettings::minDurationToSavePosition() * 60) {
         return 0;
     }
 
-    auto hash = md5(getProperty(QStringLiteral("path")).toString());
-    auto configPath = Global::instance()->appConfigDirPath();
-    KConfig *config = new KConfig(configPath.append(QStringLiteral("/watch-later/")).append(hash));
-    int pos = config->group("").readEntry("TimePosition", QString::number(0)).toDouble();
+    auto hash = md5(currentUrl().toLocalFile());
+    auto watchLaterConfig = QString(m_watchLaterPath).append(hash);
+    KConfig config(watchLaterConfig);
+    auto pos = config.group("").readEntry("TimePosition", QString::number(0)).toDouble();
 
     return pos;
 }
 
 void MpvItem::resetTimePosition()
 {
-    auto hash = md5(getProperty(QStringLiteral("path")).toString());
-    auto configPath = Global::instance()->appConfigDirPath();
-    QFile f(configPath.append(QStringLiteral("/watch-later/")).append(hash));
+    auto hash = md5(currentUrl().toLocalFile());
+    auto watchLaterConfig = QString(m_watchLaterPath).append(hash);
+    QFile f(watchLaterConfig);
 
     if (f.exists()) {
         f.remove();
@@ -639,6 +556,197 @@ QString MpvItem::md5(const QString &str)
     return QString::fromUtf8(md5.toHex());
 }
 
+PlayListModel *MpvItem::playlistModel()
+{
+    return m_playlistModel;
+}
+
+void MpvItem::setPlaylistModel(PlayListModel *model)
+{
+    m_playlistModel = model;
+}
+
+PlayListProxyModel *MpvItem::playlistProxyModel()
+{
+    return m_playlistProxyModel;
+}
+
+void MpvItem::setPlaylistProxyModel(PlayListProxyModel *model)
+{
+    m_playlistProxyModel = model;
+}
+
+ChaptersModel *MpvItem::chaptersModel() const
+{
+    return m_chaptersModel;
+}
+
+void MpvItem::setChaptersModel(ChaptersModel *_chaptersModel)
+{
+    if (m_chaptersModel == _chaptersModel) {
+        return;
+    }
+    m_chaptersModel = _chaptersModel;
+    Q_EMIT chaptersModelChanged();
+}
+
+TracksModel *MpvItem::subtitleTracksModel() const
+{
+    return m_subtitleTracksModel;
+}
+
+TracksModel *MpvItem::audioTracksModel() const
+{
+    return m_audioTracksModel;
+}
+
+bool MpvItem::isFileReloaded() const
+{
+    return m_isFileReloaded;
+}
+
+void MpvItem::setIsFileReloaded(bool _isFileReloaded)
+{
+    if (m_isFileReloaded == _isFileReloaded) {
+        return;
+    }
+    m_isFileReloaded = _isFileReloaded;
+    Q_EMIT isFileReloadedChanged();
+}
+
+QString MpvItem::mediaTitle()
+{
+    return getCachedPropertyValue(MpvProperties::self()->MediaTitle).toString();
+}
+
+double MpvItem::position()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Position).toDouble();
+}
+
+void MpvItem::setPosition(double value)
+{
+    if (qFuzzyCompare(value, position())) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Position, value);
+}
+
+double MpvItem::remaining()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Remaining).toDouble();
+}
+
+double MpvItem::duration()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Duration).toDouble();
+}
+
+bool MpvItem::pause()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Pause).toBool();
+}
+
+void MpvItem::setPause(bool value)
+{
+    if (value == pause()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Pause, value);
+}
+
+int MpvItem::volume()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Volume).toInt();
+}
+
+void MpvItem::setVolume(int value)
+{
+    if (value == volume()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Volume, value);
+}
+
+int MpvItem::volumeMax()
+{
+    return getCachedPropertyValue(MpvProperties::self()->VolumeMax).toInt();
+}
+
+void MpvItem::setVolumeMax(int value)
+{
+    if (volumeMax() == value) {
+        return;
+    }
+
+    setProperty(MpvProperties::self()->VolumeMax, value);
+}
+
+bool MpvItem::mute()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Mute).toBool();
+}
+
+void MpvItem::setMute(bool value)
+{
+    if (value == mute()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Mute, value);
+}
+
+int MpvItem::chapter()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Chapter).toInt();
+}
+
+void MpvItem::setChapter(int value)
+{
+    if (value == chapter()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Chapter, value);
+}
+
+int MpvItem::audioId()
+{
+    return getCachedPropertyValue(MpvProperties::self()->AudioId).toInt();
+}
+
+void MpvItem::setAudioId(int value)
+{
+    if (value == audioId()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->AudioId, value);
+}
+
+int MpvItem::subtitleId()
+{
+    return getCachedPropertyValue(MpvProperties::self()->SubtitleId).toInt();
+}
+
+void MpvItem::setSubtitleId(int value)
+{
+    if (value == subtitleId()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->SubtitleId, value);
+}
+
+int MpvItem::secondarySubtitleId()
+{
+    return getCachedPropertyValue(MpvProperties::self()->SecondarySubtitleId).toInt();
+}
+
+void MpvItem::setSecondarySubtitleId(int value)
+{
+    if (value == secondarySubtitleId()) {
+        return;
+    }
+    setProperty(MpvProperties::self()->SecondarySubtitleId, value);
+}
+
 QString MpvItem::formattedDuration() const
 {
     return m_formattedDuration;
@@ -654,9 +762,77 @@ QString MpvItem::formattedPosition() const
     return m_formattedPosition;
 }
 
-QString MpvItem::currentFile() const
+QUrl MpvItem::currentUrl() const
 {
-    return m_currentFile;
+    return m_currentUrl;
+}
+
+double MpvItem::watchLaterPosition() const
+{
+    return m_watchLaterPosition;
+}
+
+void MpvItem::setWatchLaterPosition(double _watchLaterPosition)
+{
+    if (qFuzzyCompare(m_watchLaterPosition + 1, _watchLaterPosition + 1)) {
+        return;
+    }
+    m_watchLaterPosition = _watchLaterPosition;
+    Q_EMIT watchLaterPositionChanged();
+}
+
+bool MpvItem::finishedLoading() const
+{
+    return m_finishedLoading;
+}
+
+void MpvItem::setFinishedLoading(bool _finishedLoading)
+{
+    if (m_finishedLoading == _finishedLoading) {
+        return;
+    }
+    m_finishedLoading = _finishedLoading;
+    Q_EMIT finishedLoadingChanged();
+}
+
+double MpvItem::watchPercentage()
+{
+    return m_watchPercentage;
+}
+
+void MpvItem::setWatchPercentage(double value)
+{
+    if (qFuzzyCompare(m_watchPercentage, value)) {
+        return;
+    }
+    m_watchPercentage = value;
+    Q_EMIT watchPercentageChanged();
+}
+
+int MpvItem::videoWidth()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Width).toInt();
+}
+
+void MpvItem::setVideoWidth(int value)
+{
+    if (width() == value) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Width, value);
+}
+
+int MpvItem::videoHeight()
+{
+    return getCachedPropertyValue(MpvProperties::self()->Height).toInt();
+}
+
+void MpvItem::setVideoHeight(int value)
+{
+    if (height() == value) {
+        return;
+    }
+    setProperty(MpvProperties::self()->Height, value);
 }
 
 #include "moc_mpvitem.cpp"
